<?php

namespace Drupal\media_contextual_crop_embed\Plugin\MediaContextualCropUseCase;

use Drupal\image\Plugin\Field\FieldType\ImageItem;
use Drupal\media_contextual_crop\MediaContextualCropUseCasePluginBase;

/**
 * Plugin implementation of the media_contextual_crop_use_case.
 *
 * @MediaContextualCropUseCase(
 *   id = "embed",
 *   label = @Translation("Embed"),
 *   default_folder = "multi_crop_embed"
 * )
 */
class Embed extends MediaContextualCropUseCasePluginBase {

  /**
   * {@inheritdoc}
   */
  public function isCompetent(ImageItem $item) {
    return (isset($item->media_contextual_crop_plugin_type) && isset($item->media_contextual_crop_embed_settings));
  }

  /**
   * {@inheritdoc}
   */
  public function processImage(&$variables) {

    $new_uri = $this->generateNewUri($variables['item'], $variables['image']['#uri'], $variables['image']['#style_name']);
    if ($new_uri != NULL) {
      $variables['image']['#uri'] = $new_uri;
    }

  }

  /**
   * {@inheritdoc}
   */
  public function processImageUrl(ImageItem $item, $old_uri, $image_style) {
    return $this->generateNewUri($item, $old_uri, $image_style);
  }

  /**
   * Generate New URI for current image.
   *
   * @param \Drupal\image\Plugin\Field\FieldType\ImageItem $item
   *   Image Item Data.
   * @param string $old_uri
   *   Uri of current image.
   * @param string $image_style
   *   Image style to apply.
   *
   * @return string|null
   *   Return new URI or NULL
   */
  private function generateNewUri(ImageItem $item, $old_uri, $image_style) {
    if ($plugin_id = $item->media_contextual_crop_plugin_type) {

      if ($embed_data = $item->media_contextual_crop_embed_settings) {

        // Load contextual Crop plugins.
        /** @var \Drupal\media_contextual_crop\MediaContextualCropInterface $plugin */
        $plugin = $this->mccPluginManager->createInstance($plugin_id);

        // Get usable CROP settings.
        $media_contextual_crop_settings = $plugin->processEmbedData($embed_data, ['image_style' => $image_style]);

        $item_values = $item->getValue();
        $old_image = [
          '#uri' => $old_uri,
          '#width' => $item_values['width'],
          '#height' => $item_values['height'],
        ];

        // Prepare Crop & generate new image path.
        $new_uri = $this->mccService->prepareCropImage($old_image, $media_contextual_crop_settings);
        return $new_uri;
      }
    }

    return NULL;
  }

}
